#
# note: in order to use these macros we must add some mine_functions:
#       network.default_route, network.interfaces
#

from __future__ import absolute_import


# note: do not import caasp modules other than caasp_log
from caasp_log import error, debug


DEFAULT_INTERFACE = 'eth0'

NODENAME_GRAIN = 'nodename'


def __virtual__():
    return "caasp_net"


def _netmask_to_cidr(netmask):
    '''
    code to convert netmask ip to cidr number
    :param netmask: netmask ip addr (eg: 255.255.255.0)
    :return: equivalent cidr number to given netmask ip (eg: 24)
    '''
    return sum([bin(int(x)).count('1') for x in netmask.split('.')])


def _calculate_subnet(ip, netmask):
    """Given an IP address and a netmask, return the subnet that IP belongs to
    :param ip: the IP address
    :param netmask: the netmask
    :return: a string with the subnet

    Example: _calculate_subnet("192.168.9.1", "255.255.0.0")
             '192.168.0.0'
    """
    return ".".join(map(str, [i & m
                              for i, m in zip(map(int, ip.split(".")),
                                              map(int, netmask.split(".")))]))


def get_hosts():
    """Wrapper around hosts.list_hosts that strips useless lines.

    hosts.list_hosts includes comments as hosts - this function strips those out.

    :return: Dict(ip_address -> [aliases])
    """
    hosts = __salt__['hosts.list_hosts']()  # type: dict
    for key in hosts.keys():
        if key.startswith("comment-"):
            hosts.pop(key)
    return hosts


def get_aliases(hostname):
    """Return the aliases for the given hostname

    :param nodename:
    :return: [string]
    """
    hosts = get_hosts()
    found_aliases = set([hostname])
    for host, aliases in hosts.iteritems():
        if hostname in aliases:
            found_aliases.update(set(aliases))
    debug('CaaS: retrieved aliases %s for %s', aliases, hostname)
    return found_aliases


def get_iface_info(iface, host=None, ifaces=None):
    '''
    given an 'iface' (and an optional 'host' and list of 'ifaces'),
    return the interface details, as a map with:
      * address (ie, 192.168.200.100)
      * broadcast (ie, 192.168.200.255)
      * label (ie, eth0)
      * netmask (ie, 255.255.255.0)
    '''
    try:
        if not ifaces:
            if not host or host == get_nodename():
                ifaces = __salt__['network.interfaces']()
            else:
                ifaces = __salt__['caasp_grains.get'](host, 'network.interfaces', type='glob')

        iface = ifaces.get(iface)
        ipv4addr = iface.get('inet', [{}])
        return ipv4addr[0]
    except Exception as e:
        error('could not get IP for interface %s: %s', iface, e)
        raise


def get_iface_ip(iface, host=None, ifaces=None):
    '''
    given an 'iface' (and an optional 'host' and list of 'ifaces'),
    return the IP address associated with 'iface'
    '''
    try:
        iface_info = get_iface_info(iface, host=host, ifaces=ifaces)
        return iface_info.get('address')
    except Exception as e:
        error('could not get IP for interface %s: %s', iface, e)
        return ''


def get_iface_subnet(iface, host=None, ifaces=None):
    '''
    given an 'iface' (and an optional 'host' and list of 'ifaces'),
    return the subnet for that interface
    '''
    try:
        iface_info = get_iface_info(iface, host=host, ifaces=ifaces)
        ip = iface_info.get('address')
        netmask = iface_info.get('netmask')
        return "{}/{}".format(_calculate_subnet(ip, netmask), _netmask_to_cidr(netmask))
    except Exception as e:
        error('could not get subnet for interface %s: %s', iface, e)
        return ''


def get_primary_iface(host=None):
    '''
    (given some optional 'host')
    return the name of the primary iface (the iface associated with the default route)
    '''
    try:
        if not host or host == get_nodename():
            default_route_lst = __salt__['network.default_route']()
            return default_route_lst[0]['interface']
        else:
            all_routes = __salt__['caasp_grains.get'](host, 'network.default_route', type='glob')
            return all_routes[host][0]['interface']
    except Exception as e:
        error('could not get the primary interface: %s', e)
        return __salt__['caasp_pillar.get']('hw:netiface', DEFAULT_INTERFACE)


def get_primary_ip(host=None, ifaces=None):
    '''
    (given an optional minion 'host' and a list of its network interfaces, 'ifaces'),
    return the primary IP
    '''
    return get_iface_ip(iface=get_primary_iface(host=host), host=host, ifaces=ifaces)


def get_primary_ips_for(compound, **kwargs):
    '''
    given a compound expression 'compound', return the primary IPs for all the
    nodes that match that expression
    '''
    try:
        all_ifaces = __salt__['caasp_grains.get'](compound, 'network.interfaces')
        return [get_primary_ip(host=host, ifaces=all_ifaces[host]) for host in all_ifaces.keys()]
    except Exception as e:
        error('could not get primary IPs for %s: %s', compound, e)
        return []


def get_primary_net(host=None, ifaces=None):
    '''
    (given some optional 'host')
    return the primary network
    '''
    return get_iface_subnet(get_primary_iface(host=host), host=host, ifaces=ifaces)


def get_primary_net_for(compound):
    '''
    given some compound expression (like, 'G@roles:kube-minion') return a list of unique
    primary networks for these nodes
    '''
    all_ifaces = __salt__['caasp_grains.get'](compound, 'network.interfaces')
    all_nets = set()
    for host in all_ifaces.keys():
        all_nets.add(get_primary_net(host, ifaces=all_ifaces[host]))
    return list(all_nets)


def get_nodename(host=None, **kwargs):
    '''
    (given some optional 'host')
    return the `nodename`
    '''
    try:
        if not host:
            assert __opts__['__role'] != 'master'
            return __salt__['grains.get'](NODENAME_GRAIN)
        else:
            all_nodenames = __salt__['caasp_grains.get'](host, grain=NODENAME_GRAIN, type='glob')
            return all_nodenames[host]
    except Exception as e:
        error('could not get nodename: %s', e)
        return ''
